/*
 * Copyright (c) 2011, Jonathan Keatley. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library.  If not, see <http://www.gnu.org/licenses/lgpl-2.1.html>.
 */
package spantable;

import javax.swing.JComponent;
import javax.swing.plaf.basic.BasicTableUI;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

public class CellSpanTableUI extends BasicTableUI {

  @Override
  public void paint(Graphics g, JComponent c) {
    Rectangle r = g.getClipBounds();
    int firstRow = table.rowAtPoint(new Point(0, r.y));
    int lastRow = table.rowAtPoint(new Point(0, r.y + r.height));
    // -1 is a flag that the ending point is outside the table
    if (lastRow < 0) lastRow = table.getRowCount() - 1;
    for (int i = firstRow; i <= lastRow; i++) {
      paintRow(i, g);
    }
  }

  private void paintRow(int row, Graphics g) {
    Rectangle r = g.getClipBounds();
    TableModel model = table.getModel();
    for (int i = 0; i < table.getColumnCount(); i++) {
      Rectangle r1 = table.getCellRect(row, i, true);
      if (r1.intersects(r)) { // at least a part is visible
        paintCell(row, i, g, r1);
      }
    }
  }

  private void paintCell(int row, int column, Graphics g, Rectangle area) {
    int verticalMargin = table.getRowMargin();
    int horizontalMargin = table.getColumnModel().getColumnMargin();

    Color c = g.getColor();
    g.setColor(table.getGridColor());
    if (table.getShowHorizontalLines()) {
        g.drawLine(area.x, area.y + area.height - 1, area.x + area.width - 1, area.y + area.height - 1);
    }
    if (table.getShowVerticalLines()) {
        g.drawLine(area.x + area.width - 1, area.y, area.x + area.width - 1, area.y + area.height - 1);
    }
    g.setColor(c);

    area.setBounds(area.x + horizontalMargin / 2, area.y + verticalMargin / 2, area.width - horizontalMargin, area.height - verticalMargin);

    if (table.isEditing() && table.getEditingRow() == row && table.getEditingColumn() == column) {
      Component component = table.getEditorComponent();
      component.setBounds(area);
      component.validate();
    }
    else {
      TableCellRenderer renderer = table.getCellRenderer(row, column);
      Component component = table.prepareRenderer(renderer, row, column);
      if (component.getParent() == null) rendererPane.add(component);
      rendererPane.paintComponent(g, component, table, area.x, area.y, area.width, area.height, true);
    }
  }
}
